#include <QMessageBox>

#include "conversationwidget.h"
#include "ui_conversationwidget.h"
#include "../helpers/conversationslisthelper.h"
#include "../helpers/conversationhelper.h"
#include "../config.h"

ConversationWidget::ConversationWidget(const Conversation &conversation, const UsersList &list, QWidget *parent) :
    QWidget(parent),
    ui(new Ui::ConversationWidget),
    mConversation(conversation),
    mUsersList(list)
{
    ui->setupUi(this);

    //Initialize UI
    QString convTitle = ConversationsListHelper::getConversationDisplayName(conversation, list);
    ui->convName->setText(convTitle);

    //Initalize helpers
    mConversationHelper = new ConversationHelper(this);
    connect(mConversationHelper, &ConversationHelper::sendMessageCallback, this, &ConversationWidget::sendMessageCallback);
}

ConversationWidget::~ConversationWidget()
{
    delete ui;
}

void ConversationWidget::sendMessage()
{
    if(isSendMessageFormLocked()){
        qInfo("sendMessage cancelled because another send message request is running...");
        return;
    }

    QString content = ui->messageContentInput->text();

    //Check message length
    if(content.length() < CONVERSATION_MESSAGE_MIN_LENGTH){
        QMessageBox::warning(this, tr("Invalid message!"), tr("Specified message is too short!"));
        return;
    }

    //Lock send form
    setSendMessageFormLocked(true);

    //Send a request to send message
    NewConversationMessage newMessage;
    newMessage.setIDConversation(mConversation.iD());
    newMessage.setMessage(content);

    //Request the message to be sent
    mConversationHelper->sendMessage(newMessage);
}

void ConversationWidget::sendMessageCallback(bool success)
{
    setSendMessageFormLocked(false);

    if(!success){
        QMessageBox::warning(this, tr("Error"), tr("Could not send your message! Please check it and your Internet connection..."));
        return;
    }

    //Reset message form
    resetSendMessageForm();
}

void ConversationWidget::on_sendMessageButton_clicked()
{
    sendMessage();
}

void ConversationWidget::on_messageContentInput_returnPressed()
{
    sendMessage();
}

void ConversationWidget::setSendMessageFormLocked(bool lock)
{
    ui->sendMessageButton->setEnabled(!lock);
    ui->messageContentInput->setEnabled(!lock);
}

bool ConversationWidget::isSendMessageFormLocked()
{
    return !ui->sendMessageButton->isEnabled();
}

void ConversationWidget::resetSendMessageForm()
{
    ui->messageContentInput->setText("");
}
